"""
Linking to standard lua library.

"""

import docutils.nodes
import sphinx.addnodes
import sphinx.application
import sphinx.environment
import sphinx.errors
import sphinx.ext.intersphinx

targets = {
    "_G": "pdf-_G",
    "_VERSION": "pdf-_VERSION",
    "assert": "pdf-assert",
    "collectgarbage": "pdf-collectgarbage",
    "dofile": "pdf-dofile",
    "error": "pdf-error",
    "getmetatable": "pdf-getmetatable",
    "ipairs": "pdf-ipairs",
    "load": "pdf-load",
    "loadfile": "pdf-loadfile",
    "next": "pdf-next",
    "pairs": "pdf-pairs",
    "pcall": "pdf-pcall",
    "print": "pdf-print",
    "rawequal": "pdf-rawequal",
    "rawget": "pdf-rawget",
    "rawlen": "pdf-rawlen",
    "rawset": "pdf-rawset",
    "require": "pdf-require",
    "select": "pdf-select",
    "setmetatable": "pdf-setmetatable",
    "tonumber": "pdf-tonumber",
    "tostring": "pdf-tostring",
    "type": "pdf-type",
    "warn": "pdf-warn",
    "xpcall": "pdf-xpcall",
    "coroutine": "6.2",
    "coroutine.close": "pdf-coroutine.close",
    "coroutine.create": "pdf-coroutine.create",
    "coroutine.isyieldable": "pdf-coroutine.isyieldable",
    "coroutine.resume": "pdf-coroutine.resume",
    "coroutine.running": "pdf-coroutine.running",
    "coroutine.status": "pdf-coroutine.status",
    "coroutine.wrap": "pdf-coroutine.wrap",
    "coroutine.yield": "pdf-coroutine.yield",
    "debug": "6.10",
    "debug.debug": "pdf-debug.debug",
    "debug.gethook": "pdf-debug.gethook",
    "debug.getinfo": "pdf-debug.getinfo",
    "debug.getlocal": "pdf-debug.getlocal",
    "debug.getmetatable": "pdf-debug.getmetatable",
    "debug.getregistry": "pdf-debug.getregistry",
    "debug.getupvalue": "pdf-debug.getupvalue",
    "debug.getuservalue": "pdf-debug.getuservalue",
    "debug.sethook": "pdf-debug.sethook",
    "debug.setlocal": "pdf-debug.setlocal",
    "debug.setmetatable": "pdf-debug.setmetatable",
    "debug.setupvalue": "pdf-debug.setupvalue",
    "debug.setuservalue": "pdf-debug.setuservalue",
    "debug.traceback": "pdf-debug.traceback",
    "debug.upvalueid": "pdf-debug.upvalueid",
    "debug.upvaluejoin": "pdf-debug.upvaluejoin",
    "io": "6.8",
    "io.close": "pdf-io.close",
    "io.flush": "pdf-io.flush",
    "io.input": "pdf-io.input",
    "io.lines": "pdf-io.lines",
    "io.open": "pdf-io.open",
    "io.output": "pdf-io.output",
    "io.popen": "pdf-io.popen",
    "io.read": "pdf-io.read",
    "io.stderr": "pdf-io.stderr",
    "io.stdin": "pdf-io.stdin",
    "io.stdout": "pdf-io.stdout",
    "io.tmpfile": "pdf-io.tmpfile",
    "io.type": "pdf-io.type",
    "io.write": "pdf-io.write",
    "file.close": "pdf-file:close",
    "file.flush": "pdf-file:flush",
    "file.lines": "pdf-file:lines",
    "file.read": "pdf-file:read",
    "file.seek": "pdf-file:seek",
    "file.setvbuf": "pdf-file:setvbuf",
    "file.write": "pdf-file:write",
    "math": "6.7",
    "math.abs": "pdf-math.abs",
    "math.acos": "pdf-math.acos",
    "math.asin": "pdf-math.asin",
    "math.atan": "pdf-math.atan",
    "math.ceil": "pdf-math.ceil",
    "math.cos": "pdf-math.cos",
    "math.deg": "pdf-math.deg",
    "math.exp": "pdf-math.exp",
    "math.floor": "pdf-math.floor",
    "math.fmod": "pdf-math.fmod",
    "math.huge": "pdf-math.huge",
    "math.log": "pdf-math.log",
    "math.max": "pdf-math.max",
    "math.maxinteger": "pdf-math.maxinteger",
    "math.min": "pdf-math.min",
    "math.mininteger": "pdf-math.mininteger",
    "math.modf": "pdf-math.modf",
    "math.pi": "pdf-math.pi",
    "math.rad": "pdf-math.rad",
    "math.random": "pdf-math.random",
    "math.randomseed": "pdf-math.randomseed",
    "math.sin": "pdf-math.sin",
    "math.sqrt": "pdf-math.sqrt",
    "math.tan": "pdf-math.tan",
    "math.tointeger": "pdf-math.tointeger",
    "math.type": "pdf-math.type",
    "math.ult": "pdf-math.ult",
    "os": "6.9",
    "os.clock": "pdf-os.clock",
    "os.date": "pdf-os.date",
    "os.difftime": "pdf-os.difftime",
    "os.execute": "pdf-os.execute",
    "os.exit": "pdf-os.exit",
    "os.getenv": "pdf-os.getenv",
    "os.remove": "pdf-os.remove",
    "os.rename": "pdf-os.rename",
    "os.setlocale": "pdf-os.setlocale",
    "os.time": "pdf-os.time",
    "os.tmpname": "pdf-os.tmpname",
    "package": "6.3",
    "package.config": "pdf-package.config",
    "package.cpath": "pdf-package.cpath",
    "package.loaded": "pdf-package.loaded",
    "package.loadlib": "pdf-package.loadlib",
    "package.path": "pdf-package.path",
    "package.preload": "pdf-package.preload",
    "package.searchers": "pdf-package.searchers",
    "package.searchpath": "pdf-package.searchpath",
    # "string": "6.4",
    "string.byte": "pdf-string.byte",
    "string.char": "pdf-string.char",
    "string.dump": "pdf-string.dump",
    "string.find": "pdf-string.find",
    "string.format": "pdf-string.format",
    "string.gmatch": "pdf-string.gmatch",
    "string.gsub": "pdf-string.gsub",
    "string.len": "pdf-string.len",
    "string.lower": "pdf-string.lower",
    "string.match": "pdf-string.match",
    "string.pack": "pdf-string.pack",
    "string.packsize": "pdf-string.packsize",
    "string.rep": "pdf-string.rep",
    "string.reverse": "pdf-string.reverse",
    "string.sub": "pdf-string.sub",
    "string.unpack": "pdf-string.unpack",
    "string.upper": "pdf-string.upper",
    # "table": "6.6",
    "table.concat": "pdf-table.concat",
    "table.insert": "pdf-table.insert",
    "table.move": "pdf-table.move",
    "table.pack": "pdf-table.pack",
    "table.remove": "pdf-table.remove",
    "table.sort": "pdf-table.sort",
    "table.unpack": "pdf-table.unpack",
    "utf8": "6.5",
    "utf8.char": "pdf-utf8.char",
    "utf8.charpattern": "pdf-utf8.charpattern",
    "utf8.codepoint": "pdf-utf8.codepoint",
    "utf8.codes": "pdf-utf8.codes",
    "utf8.len": "pdf-utf8.len",
    "utf8.offset": "pdf-utf8.offset",
    # "getfenv": "pdf-getfenv",
    # "loadstring": "pdf-loadstring",
    # "module": "pdf-module",
    # "setfenv": "pdf-setfenv",
    # "unpack": "pdf-unpack",
    # "debug.getfenv": "pdf-debug.getfenv",
    # "debug.setfenv": "pdf-debug.setfenv",
    # "math.atan2": "pdf-math.atan2",
    # "math.cosh": "pdf-math.cosh",
    # "math.frexp": "pdf-math.frexp",
    # "math.ldexp": "pdf-math.ldexp",
    # "math.log10": "pdf-math.log10",
    # "math.pow": "pdf-math.pow",
    # "math.sinh": "pdf-math.sinh",
    # "math.tanh": "pdf-math.tanh",
    # "package.loaders": "pdf-package.loaders",
    # "package.seeall": "pdf-package.seeall",
    # "table.maxn": "pdf-table.maxn",
}


def resolve_std_reference(
    app: sphinx.application.Sphinx,
    env: sphinx.environment.BuildEnvironment,
    node: sphinx.addnodes.pending_xref,
    contnode: docutils.nodes.Node,
):
    if node["refdomain"] == "lua" or node["reftype"] == "any":
        target = node["reftarget"]
        if target in targets:
            lua = env.config["lua_ls_lua_version"]
            uri = f"https://www.lua.org/manual/{lua}/manual.html#{targets[target]}"
            ref = docutils.nodes.reference("", "", contnode)
            ref["refuri"] = uri
            return ref
